import itertools, collections
import numpy as np

class dictee(dict):
    """
    反转时行为
    ----------
    要求 value 可 hash。否则请用 invert_unhashable。
    value 重复时合并，取最后出现的 key。np.nan 重复也会合并。
    输出类型仍是 idict。

    固化顺序
    --------
    输出为嵌套元组

    example
    -------
    vd = idict(name='Allen', age=np.nan, gender='male', gende=np.nan) 
    vd.invert
    vd.invert_unhashable
    vd.preserve
    vd.invert.preserve
    vd.invert_unhashable.preserve
    """
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
    @property
    def invert(self):
        return self.__class__({v: k for k, v in self.items()})
    @property
    def invert_unhashable(self):
        return self.__class__({str(v): k for k, v in self.items()})
    @property
    def preserve(self):
        return tuple(self.items())



import itertools, collections
import numpy as np

def counteree(data):
    """
    description
    -----------
    collections.Counter 计算纯数字的pd.Series时，会出现np.nan无法合并统计。
    """
    data_list = []
    for i in data:
        if isinstance(i, float) and np.isnan(i):  # np.isnan just 只能判断数值，且对于np.array会广播，所以要限制类型。
            data_list.append(np.nan)
        else:
            data_list.append(i)
    return collections.Counter(data_list)

def chainee(datas):
    """
    description
    -----------
    原始chain只能处理iterable对象，这里可以自动将非iterable对象放入列表，然后进行chain。
    """
    dt = [x if isinstance(x, collections.Iterable) else [x] for x in datas]
    rst = itertools.chain(*dt)
    return rst



from setuptools import setup, find_packages
setup(
    name = "whlee",
    version = "0.0.2",
    description="toy tools update at 20250516",
    author = "whl",
    author_email = "2631@139.com",
    license="MIT",
    packages = find_packages(),
    python_requires=">=3.8",
    install_requires=['numpy==1.24.3', 'pandas>=1.3.5', 'addict']
)
